var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", '@angular/core', '../../config/config', './swiper/swiper-keyboard', '../ion', '../../util/util', './swiper/swiper-events', './swiper/swiper-zoom', '../../platform/platform', './swiper/swiper', './swiper/swiper-effects', '../../navigation/view-controller'], factory);
    }
})(function (require, exports) {
    "use strict";
    var core_1 = require('@angular/core');
    var config_1 = require('../../config/config');
    var swiper_keyboard_1 = require('./swiper/swiper-keyboard');
    var ion_1 = require('../ion');
    var util_1 = require('../../util/util');
    var swiper_events_1 = require('./swiper/swiper-events');
    var swiper_zoom_1 = require('./swiper/swiper-zoom');
    var platform_1 = require('../../platform/platform');
    var swiper_1 = require('./swiper/swiper');
    var swiper_effects_1 = require('./swiper/swiper-effects');
    var view_controller_1 = require('../../navigation/view-controller');
    /**
     * @name Slides
     * @description
     * The Slides component is a multi-section container. Each section can be swiped
     * or dragged between. It contains any number of [Slide](../Slide) components.
     *
     *
     * ### Creating
     * You should use a template to create slides and listen to slide events. The template
     * should contain the slide container, an `<ion-slides>` element, and any number of
     * [Slide](../Slide) components, written as `<ion-slide>`. Basic configuration
     * values can be set as input properties, which are listed below. Slides events
     * can also be listened to such as the slide changing by placing the event on the
     * `<ion-slides>` element. See [Usage](#usage) below for more information.
     *
     *
     * ### Navigating
     * After creating and configuring the slides, you can navigate between them
     * by swiping or calling methods on the `Slides` instance. You can call `slideTo()` to
     * navigate to a specific slide, or `slideNext()` to change to the slide that follows
     * the active slide. All of the [methods](#instance-members) provided by the `Slides`
     * instance are listed below. See [Usage](#usage) below for more information on
     * navigating between slides.
     *
     *
     * @usage
     *
     * You can add slides to a `@Component` using the following template:
     *
     * ```html
     * <ion-slides>
     *   <ion-slide>
     *     <h1>Slide 1</h1>
     *   </ion-slide>
     *   <ion-slide>
     *     <h1>Slide 2</h1>
     *   </ion-slide>
     *   <ion-slide>
     *     <h1>Slide 3</h1>
     *   </ion-slide>
     * </ion-slides>
     * ```
     *
     * Next, we can use `ViewChild` to assign the Slides instance to
     * your `slides` property. Now we can call any of the `Slides`
     * [methods](#instance-members), for example we can use the Slide's
     * `slideTo()` method in order to navigate to a specific slide on
     * a button click. Below we call the `goToSlide()` method and it
     * navigates to the 3rd slide:
     *
     * ```ts
     * import { ViewChild } from '@angular/core';
     *
     * class MyPage {
     *   @ViewChild(Slides) slides: Slides;
     *
     *   goToSlide() {
     *     this.slides.slideTo(2, 500);
     *   }
     * }
     * ```
     *
     * We can also add events to listen to on the `<ion-slides>` element.
     * Let's add the `ionSlideDidChange` event and call a method when the slide changes:
     *
     * ```html
     * <ion-slides (ionSlideDidChange)="slideChanged()">
     * ```
     *
     * In our class, we add the `slideChanged()` method which gets the active
     * index and prints it:
     *
     * ```ts
     * class MyPage {
     *   ...
     *
     *   slideChanged() {
     *     let currentIndex = this.slides.getActiveIndex();
     *     console.log("Current index is", currentIndex);
     *   }
     * }
     * ```
     *
     * @demo /docs/v2/demos/src/slides/
     * @see {@link /docs/v2/components#slides Slides Component Docs}
     *
     * Adopted from Swiper.js:
     * The most modern mobile touch slider and framework with
     * hardware accelerated transitions.
     *
     * http://www.idangero.us/swiper/
     *
     * Copyright 2016, Vladimir Kharlampidi
     * The iDangero.us
     * http://www.idangero.us/
     *
     * Licensed under MIT
     */
    var Slides = (function (_super) {
        __extends(Slides, _super);
        function Slides(config, _plt, viewCtrl, elementRef, renderer) {
            var _this = this;
            _super.call(this, config, elementRef, renderer, 'slides');
            this._plt = _plt;
            this._effectName = 'slide';
            this._direction = 'horizontal';
            this._initialSlide = 0;
            this._isLoop = false;
            this._pager = false;
            this._paginationType = 'bullets';
            this._isParallax = false;
            this._speedMs = 300;
            this._isZoom = false;
            /**
             * Enabled this option and swiper will be operated as usual except it will
             * not move, real translate values on wrapper will not be set. Useful when
             * you may need to create custom slide transition.
             */
            this.virtualTranslate = false;
            /**
             * Set to true to round values of slides width and height to prevent blurry
             * texts on usual resolution screens (if you have such)
             */
            this.roundLengths = false;
            // Slides grid
            this.spaceBetween = 0;
            this.slidesPerView = 1;
            this.slidesPerColumn = 1;
            this.slidesPerColumnFill = 'column';
            this.slidesPerGroup = 1;
            this.centeredSlides = false;
            this.slidesOffsetBefore = 0;
            this.slidesOffsetAfter = 0;
            // autoplay
            this.autoplayDisableOnInteraction = true;
            this.autoplayStopOnLast = false;
            // Free mode
            this.freeMode = false;
            this.freeModeMomentum = true;
            this.freeModeMomentumRatio = 1;
            this.freeModeMomentumBounce = true;
            this.freeModeMomentumBounceRatio = 1;
            this.freeModeMomentumVelocityRatio = 1;
            this.freeModeSticky = false;
            this.freeModeMinimumVelocity = 0.02;
            // Autoheight
            this.autoHeight = false;
            // Set wrapper width
            this.setWrapperSize = false;
            // Zoom
            this.zoomMax = 3;
            this.zoomMin = 1;
            this.zoomToggle = true;
            // Touches
            this.touchRatio = 1;
            this.touchAngle = 45;
            this.simulateTouch = true;
            this.shortSwipes = true;
            this.longSwipes = true;
            this.longSwipesRatio = 0.5;
            this.longSwipesMs = 300;
            this.followFinger = true;
            this.onlyExternal = false;
            this.threshold = 0;
            this.touchMoveStopPropagation = true;
            this.touchReleaseOnEdges = false;
            // To support iOS's swipe-to-go-back gesture (when being used in-app, with UIWebView).
            this.iOSEdgeSwipeDetection = false;
            this.iOSEdgeSwipeThreshold = 20;
            // Pagination
            this.paginationClickable = false;
            this.paginationHide = false;
            // Resistance
            this.resistance = true;
            this.resistanceRatio = 0.85;
            // Progress
            this.watchSlidesProgress = false;
            this.watchSlidesVisibility = false;
            // Clicks
            this.preventClicks = true;
            this.preventClicksPropagation = true;
            this.slideToClickedSlide = false;
            // loop
            this.loopAdditionalSlides = 0;
            this.loopedSlides = null;
            // Swiping/no swiping
            this.swipeHandler = null;
            this.noSwiping = true;
            // Callbacks
            this.runCallbacksOnInit = true;
            // Keyboard
            this.keyboardControl = true;
            // Effects
            this.coverflow = {
                rotate: 50,
                stretch: 0,
                depth: 100,
                modifier: 1,
                slideShadows: true
            };
            this.flip = {
                slideShadows: true,
                limitRotation: true
            };
            this.cube = {
                slideShadows: true,
                shadow: true,
                shadowOffset: 20,
                shadowScale: 0.94
            };
            this.fade = {
                crossFade: false
            };
            // Accessibility
            this.prevSlideMessage = 'Previous slide';
            this.nextSlideMessage = 'Next slide';
            this.firstSlideMessage = 'This is the first slide';
            this.lastSlideMessage = 'This is the last slide';
            /**
             * @output {Slides} Expression to evaluate when a slide change starts.
             */
            this.ionSlideWillChange = new core_1.EventEmitter();
            /**
             * @output {Slides} Expression to evaluate when a slide change ends.
             */
            this.ionSlideDidChange = new core_1.EventEmitter();
            /**
             * @output {Slides} Expression to evaluate when a slide moves.
             */
            this.ionSlideDrag = new core_1.EventEmitter();
            /**
             * @output {Slides} When slides reach its beginning (initial position).
             */
            this.ionSlideReachStart = new core_1.EventEmitter();
            /**
             * @output {Slides} When slides reach its last slide.
             */
            this.ionSlideReachEnd = new core_1.EventEmitter();
            /**
             * @output {Slides} Expression to evaluate when a slide moves.
             */
            this.ionSlideAutoplay = new core_1.EventEmitter();
            /**
             * @output {Slides} Same as `ionWillChange` but caused by autoplay.
             */
            this.ionSlideAutoplayStart = new core_1.EventEmitter();
            /**
             * @output {Slides} Expression to evaluate when a autoplay stops.
             */
            this.ionSlideAutoplayStop = new core_1.EventEmitter();
            /**
             * @output {Slides} Same as `ionWillChange` but for "forward" direction only.
             */
            this.ionSlideNextStart = new core_1.EventEmitter();
            /**
             * @output {Slides} Same as `ionWillChange` but for "backward" direction only.
             */
            this.ionSlidePrevStart = new core_1.EventEmitter();
            /**
             * @output {Slides} Same as `ionDidChange` but for "forward" direction only.
             */
            this.ionSlideNextEnd = new core_1.EventEmitter();
            /**
             * @output {Slides} Same as `ionDidChange` but for "backward" direction only.
             */
            this.ionSlidePrevEnd = new core_1.EventEmitter();
            /**
             * @output {Slides} When the user taps/clicks on the slide's container.
             */
            this.ionSlideTap = new core_1.EventEmitter();
            /**
             * @output {Slides} When the user double taps on the slide's container.
             */
            this.ionSlideDoubleTap = new core_1.EventEmitter();
            /** @private */
            this.ionSlideProgress = new core_1.EventEmitter();
            /** @private */
            this.ionSlideTransitionStart = new core_1.EventEmitter();
            /** @private */
            this.ionSlideTransitionEnd = new core_1.EventEmitter();
            /** @private */
            this.ionSlideTouchStart = new core_1.EventEmitter();
            /** @private */
            this.ionSlideTouchEnd = new core_1.EventEmitter();
            this._unregs = [];
            /** @internal */
            this._allowSwipeToNext = true;
            /** @internal */
            this._allowSwipeToPrev = true;
            this.id = ++slidesId;
            this.slideId = 'slides-' + this.id;
            this.setElementClass(this.slideId, true);
            // only initialize the slides whent the content is ready
            if (viewCtrl) {
                var subscription = viewCtrl.readReady.subscribe(function () {
                    subscription.unsubscribe();
                    _this._initSlides();
                });
            }
        }
        Object.defineProperty(Slides.prototype, "autoplay", {
            /**
             * @input {number}  Delay between transitions (in milliseconds). If this
             * parameter is not passed, autoplay is disabled. Default does
             * not have a value and does not autoplay.
             */
            get: function () {
                return this._autoplayMs;
            },
            set: function (val) {
                this._autoplayMs = parseInt(val, 10);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "effect", {
            /**
             * @input {string} Could be `slide`, `fade`, `cube`, `coverflow` or `flip`.
             * Default: `slide`.
             */
            get: function () {
                return this._effectName;
            },
            set: function (effectName) {
                if (swiper_effects_1.SWIPER_EFFECTS[effectName]) {
                    this._effectName = effectName;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "direction", {
            /**
             * @input {string}  Swipe direction: 'horizontal' or 'vertical'.
             * Default: `horizontal`.
             */
            get: function () {
                return this._direction;
            },
            set: function (val) {
                if (val === 'horizontal' || val === 'vertical') {
                    this._direction = val;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "initialSlide", {
            /**
             * @input {number}  Index number of initial slide. Default: `0`.
             */
            get: function () {
                return this._initialSlide;
            },
            set: function (val) {
                this._initialSlide = parseInt(val, 10);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "loop", {
            /**
             * @input {boolean}  Whether to continuously loop from the last slide to the
             * first slide. Default: `false`.
             */
            get: function () {
                return this._isLoop;
            },
            set: function (val) {
                this._isLoop = util_1.isTrueProperty(val);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "pager", {
            /**
             * @input {boolean}  String with type of pagination. Can be
             * `bullets`, `fraction`, `progress`. Default does not have
             * pagination set.
             */
            get: function () {
                return this._pager;
            },
            set: function (val) {
                this._pager = util_1.isTrueProperty(val);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "paginationType", {
            /**
             * @input {boolean}  String with type of pagination. Can be
             * `bullets`, `fraction`, `progress`. Default: `bullets`.
             * (Note that the pager will not show unless `pager` input
             * is set to true).
             */
            get: function () {
                return this._paginationType;
            },
            set: function (val) {
                if (val === 'bullets' || val === 'fraction' || val === 'progress') {
                    this._paginationType = val;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "parallax", {
            /**
             * Enable, if you want to use "parallaxed" elements inside of
             * slider. Default: `false`.
             */
            get: function () {
                return this._isParallax;
            },
            set: function (val) {
                this._isParallax = util_1.isTrueProperty(val);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "speed", {
            /**
             * @input {number}  Duration of transition between slides
             * (in milliseconds). Default: `300`.
             */
            get: function () {
                return this._speedMs;
            },
            set: function (val) {
                this._speedMs = parseInt(val, 10);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "zoom", {
            /**
             * @input {boolean}  Set to `true` to enable zooming functionality.
             * Default: `false`.
             */
            get: function () {
                return this._isZoom;
            },
            set: function (val) {
                this._isZoom = util_1.isTrueProperty(val);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "options", {
            /**
             * @private
             * Deprecated
             */
            set: function (val) {
                // Deprecated warning added 2016-12-28
                console.warn('ion-slides "options" has been deprecated. Please use ion-slide\'s input properties instead.');
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "ionWillChange", {
            /**
             * @private
             * Deprecated: Use "ionSlideWillChange" instead.
             * Added 2016-12-29
             */
            get: function () {
                console.warn('ion-slides "ionWillChange" has been deprecated, please use "ionSlideWillChange" instead.');
                return new core_1.EventEmitter();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "ionDidChange", {
            /**
             * @private
             * Deprecated: Use "ionSlideWillChange" instead.
             * Added 2016-12-29
             */
            get: function () {
                console.warn('ion-slides "ionDidChange" has been deprecated, please use "ionSlideDidChange" instead.');
                return new core_1.EventEmitter();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Slides.prototype, "ionDrag", {
            /**
             * @private
             * Deprecated: Use "ionSlideWillChange" instead.
             * Added 2016-12-29
             */
            get: function () {
                console.warn('ion-slides "ionDrag" has been deprecated, please use "ionSlideDrag" instead.');
                return new core_1.EventEmitter();
            },
            enumerable: true,
            configurable: true
        });
        Slides.prototype._initSlides = function () {
            if (!this._init) {
                (void 0) /* console.debug */;
                var s = this;
                var plt = s._plt;
                s.container = this.getNativeElement().children[0];
                // init swiper core
                swiper_1.initSwiper(s, plt);
                // init core event listeners
                this._unregs.push(swiper_events_1.initEvents(s, plt));
                if (this.zoom) {
                    // init zoom event listeners
                    this._unregs.push(swiper_zoom_1.initZoom(s, plt));
                }
                if (this.keyboardControl) {
                    // init keyboard event listeners
                    s.enableKeyboardControl(true);
                }
                this._init = true;
            }
        };
        /**
         * @private
         */
        Slides.prototype.ngAfterContentInit = function () {
            var _this = this;
            this._plt.timeout(function () {
                _this._initSlides();
            }, 300);
        };
        /**
         * @private
         * Update the underlying slider implementation. Call this if you've added or removed
         * child slides.
         */
        Slides.prototype.update = function (debounce) {
            var _this = this;
            if (debounce === void 0) { debounce = 300; }
            if (this._init) {
                this._plt.cancelTimeout(this._tmr);
                this._tmr = this._plt.timeout(function () {
                    swiper_1.update(_this, _this._plt);
                    // Don't allow pager to show with > 10 slides
                    if (_this.length() > 10) {
                        _this.paginationType = undefined;
                    }
                }, debounce);
            }
        };
        /**
         * Transition to the specified slide.
         *
         * @param {number} index  The index number of the slide.
         * @param {number} [speed]  Transition duration (in ms).
         * @param {boolean} [runCallbacks] Whether or not to emit the `ionWillChange`/`ionDidChange` events. Default true.
         */
        Slides.prototype.slideTo = function (index, speed, runCallbacks) {
            swiper_1.slideTo(this, this._plt, index, speed, runCallbacks);
        };
        /**
         * Transition to the next slide.
         *
         * @param {number} [speed]  Transition duration (in ms).
         * @param {boolean} [runCallbacks]  Whether or not to emit the `ionWillChange`/`ionDidChange` events. Default true.
         */
        Slides.prototype.slideNext = function (speed, runCallbacks) {
            swiper_1.slideNext(this, this._plt, runCallbacks, speed, true);
        };
        /**
         * Transition to the previous slide.
         *
         * @param {number} [speed]  Transition duration (in ms).
         * @param {boolean} [runCallbacks]  Whether or not to emit the `ionWillChange`/`ionDidChange` events. Default true.
         */
        Slides.prototype.slidePrev = function (speed, runCallbacks) {
            swiper_1.slidePrev(this, this._plt, runCallbacks, speed, true);
        };
        /**
         * Get the index of the active slide.
         *
         * @returns {number} The index number of the current slide.
         */
        Slides.prototype.getActiveIndex = function () {
            return this._activeIndex;
        };
        /**
         * Get the index of the previous slide.
         *
         * @returns {number} The index number of the previous slide.
         */
        Slides.prototype.getPreviousIndex = function () {
            return this._previousIndex;
        };
        /**
         * Get the total number of slides.
         *
         * @returns {number} The total number of slides.
         */
        Slides.prototype.length = function () {
            return this._slides.length;
        };
        /**
         * Get whether or not the current slide is the last slide.
         *
         * @returns {boolean} If the slide is the last slide or not.
         */
        Slides.prototype.isEnd = function () {
            return this._isEnd;
        };
        /**
         * Get whether or not the current slide is the first slide.
         *
         * @returns {boolean} If the slide is the first slide or not.
         */
        Slides.prototype.isBeginning = function () {
            return this._isBeginning;
        };
        /**
         * Start auto play.
         */
        Slides.prototype.startAutoplay = function () {
            swiper_1.startAutoplay(this, this._plt);
        };
        /**
         * Stop auto play.
         */
        Slides.prototype.stopAutoplay = function () {
            swiper_1.stopAutoplay(this);
        };
        /**
         * Lock or unlock the ability to slide to the next slides.
         */
        Slides.prototype.lockSwipeToNext = function (shouldLockSwipeToNext) {
            this._allowSwipeToNext = !shouldLockSwipeToNext;
        };
        /**
         * Lock or unlock the ability to slide to the previous slides.
         */
        Slides.prototype.lockSwipeToPrev = function (shouldLockSwipeToPrev) {
            this._allowSwipeToPrev = !shouldLockSwipeToPrev;
        };
        /**
         * Lock or unlock the ability to slide to change slides.
         */
        Slides.prototype.lockSwipes = function (shouldLockSwipes) {
            this._allowSwipeToNext = this._allowSwipeToPrev = !shouldLockSwipes;
        };
        /**
         * Enable or disable keyboard control.
         */
        Slides.prototype.enableKeyboardControl = function (shouldEnableKeyboard) {
            swiper_keyboard_1.enableKeyboardControl(this, this._plt, shouldEnableKeyboard);
        };
        /**
         * @private
         */
        Slides.prototype.ngOnDestroy = function () {
            this._init = false;
            this._unregs.forEach(function (unReg) {
                unReg();
            });
            this._unregs.length = 0;
            swiper_1.destroySwiper(this);
            this.enableKeyboardControl(false);
        };
        /**
         * @private
         * Deprecated, please use the instance of ion-slides.
         */
        Slides.prototype.getSlider = function () {
            // deprecated 2016-12-29
            console.warn("ion-slides, getSlide() has been removed. Please use the properties and methods on the instance of ion-slide instead.");
        };
        Slides.decorators = [
            { type: core_1.Component, args: [{
                        selector: 'ion-slides',
                        template: '<div class="swiper-container">' +
                            '<div class="swiper-wrapper">' +
                            '<ng-content></ng-content>' +
                            '</div>' +
                            '<div [class.hide]="!pager" class="swiper-pagination"></div>' +
                            '</div>',
                        changeDetection: core_1.ChangeDetectionStrategy.OnPush,
                        encapsulation: core_1.ViewEncapsulation.None,
                    },] },
        ];
        /** @nocollapse */
        Slides.ctorParameters = [
            { type: config_1.Config, },
            { type: platform_1.Platform, },
            { type: view_controller_1.ViewController, decorators: [{ type: core_1.Optional },] },
            { type: core_1.ElementRef, },
            { type: core_1.Renderer, },
        ];
        Slides.propDecorators = {
            'autoplay': [{ type: core_1.Input },],
            'effect': [{ type: core_1.Input },],
            'direction': [{ type: core_1.Input },],
            'initialSlide': [{ type: core_1.Input },],
            'loop': [{ type: core_1.Input },],
            'pager': [{ type: core_1.Input },],
            'paginationType': [{ type: core_1.Input },],
            'parallax': [{ type: core_1.Input },],
            'speed': [{ type: core_1.Input },],
            'zoom': [{ type: core_1.Input },],
            'ionSlideWillChange': [{ type: core_1.Output },],
            'ionSlideDidChange': [{ type: core_1.Output },],
            'ionSlideDrag': [{ type: core_1.Output },],
            'ionSlideReachStart': [{ type: core_1.Output },],
            'ionSlideReachEnd': [{ type: core_1.Output },],
            'ionSlideAutoplay': [{ type: core_1.Output },],
            'ionSlideAutoplayStart': [{ type: core_1.Output },],
            'ionSlideAutoplayStop': [{ type: core_1.Output },],
            'ionSlideNextStart': [{ type: core_1.Output },],
            'ionSlidePrevStart': [{ type: core_1.Output },],
            'ionSlideNextEnd': [{ type: core_1.Output },],
            'ionSlidePrevEnd': [{ type: core_1.Output },],
            'ionSlideTap': [{ type: core_1.Output },],
            'ionSlideDoubleTap': [{ type: core_1.Output },],
            'options': [{ type: core_1.Input },],
            'ionDidChange': [{ type: core_1.Output },],
        };
        return Slides;
    }(ion_1.Ion));
    exports.Slides = Slides;
    var slidesId = -1;
});
//# sourceMappingURL=slides.js.map